@@ -1,25 +1,16 @@
 /*
- * "$Id: lpd.c 6061 2006-10-23 00:26:52Z mike $"
+ * "$Id: lpd.c 8731 2009-06-26 18:26:36Z mike $"
  *
  *   Line Printer Daemon backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -38,18 +29,11 @@
  * Include necessary headers.
  */
 
-#include <cups/backend.h>
 #include <cups/http-private.h>
-#include <cups/cups.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include "backend-private.h"
 #include <stdarg.h>
-#include <ctype.h>
-#include <cups/string.h>
-#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <signal.h>
 
 #ifdef WIN32
 #  include <winsock.h>
@@ -73,6 +57,14 @@ static char	tmpfilename[1024] = "";	/* Temporary spool file name */
 static int	abort_job = 0;		/* Non-zero if we get SIGTERM */
 
 
+/*
+ * Print mode...
+ */
+
+#define MODE_STANDARD		0	/* Queue a copy */
+#define MODE_STREAM		1	/* Stream a copy */
+
+
 /*
  * The order for control and data files in LPD requests...
  */
@@ -96,9 +88,9 @@ static int	abort_job = 0;		/* Non-zero if we get SIGTERM */
 
 static int	lpd_command(int lpd_fd, int timeout, char *format, ...);
 static int	lpd_queue(const char *hostname, int port, const char *printer,
-		          const char *filename,
-		          const char *user, const char *title, int copies,
-			  int banner, int format, int order, int reserve,
+		          int print_fd, int mode, const char *user,
+			  const char *title, int copies, int banner,
+			  int format, int order, int reserve,
 			  int manual_copies, int timeout, int contimeout);
 static void	lpd_timeout(int sig);
 static int	lpd_write(int lpd_fd, char *buffer, int length);
@@ -120,29 +112,32 @@ int					/* O - Exit status */
 main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
      char *argv[])			/* I - Command-line arguments */
 {
-  char			method[255],	/* Method in URI */
-			hostname[1024],	/* Hostname */
-			username[255],	/* Username info */
-			resource[1024],	/* Resource info (printer name) */
-			*options,	/* Pointer to options */
-			name[255],	/* Name of option */
-			value[255],	/* Value of option */
-			*ptr,		/* Pointer into name or value */
-			*filename,	/* File to print */
-			title[256];	/* Title string */
-  int			port;		/* Port number */
-  int			status;		/* Status of LPD job */
-  int			banner;		/* Print banner page? */
-  int			format;		/* Print format */
-  int			order;		/* Order of control/data files */
-  int			reserve;	/* Reserve priviledged port? */
-  int			sanitize_title;	/* Sanitize title string? */
-  int			manual_copies,	/* Do manual copies? */
-			timeout,	/* Timeout */
-			contimeout,	/* Connection timeout */
-			copies;		/* Number of copies */
+  const char	*device_uri;		/* Device URI */
+  char		scheme[255],		/* Scheme in URI */
+		hostname[1024],		/* Hostname */
+		username[255],		/* Username info */
+		resource[1024],		/* Resource info (printer name) */
+		*options,		/* Pointer to options */
+		*name,			/* Name of option */
+		*value,			/* Value of option */
+		sep,			/* Separator character */
+		*filename,		/* File to print */
+		title[256];		/* Title string */
+  int		port;			/* Port number */
+  int		fd;			/* Print file */
+  int		status;			/* Status of LPD job */
+  int		mode;			/* Print mode */
+  int		banner;			/* Print banner page? */
+  int		format;			/* Print format */
+  int		order;			/* Order of control/data files */
+  int		reserve;		/* Reserve priviledged port? */
+  int		sanitize_title;		/* Sanitize title string? */
+  int		manual_copies,		/* Do manual copies? */
+		timeout,		/* Timeout */
+		contimeout,		/* Connection timeout */
+		copies;			/* Number of copies */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction	action;		/* Actions for POSIX signals */
+  struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
@@ -179,63 +174,32 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
   if (argc == 1)
   {
-    puts("network lpd \"Unknown\" \"LPD/LPR Host or Printer\"");
+    printf("network lpd \"Unknown\" \"%s\"\n",
+           _cupsLangString(cupsLangDefault(), _("LPD/LPR Host or Printer")));
     return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
-    fprintf(stderr, "Usage: %s job-id user title copies options [file]\n",
-            argv[0]);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]\n"),
+                    argv[0]);
     return (CUPS_BACKEND_FAILED);
   }
 
- /*
-  * If we have 7 arguments, print the file named on the command-line.
-  * Otherwise, copy stdin to a temporary file and print the temporary
-  * file.
-  */
-
-  if (argc == 6)
-  {
-   /*
-    * Copy stdin to a temporary file...
-    */
-
-    int  fd;		/* Temporary file */
-    char buffer[8192];	/* Buffer for copying */
-    int  bytes;		/* Number of bytes read */
-
-
-    if ((fd = cupsTempFd(tmpfilename, sizeof(tmpfilename))) < 0)
-    {
-      perror("ERROR: unable to create temporary file");
-      return (CUPS_BACKEND_FAILED);
-    }
-
-    while ((bytes = fread(buffer, 1, sizeof(buffer), stdin)) > 0)
-      if (write(fd, buffer, bytes) < bytes)
-      {
-        perror("ERROR: unable to write to temporary file");
-	close(fd);
-	unlink(tmpfilename);
-	return (CUPS_BACKEND_FAILED);
-      }
-
-    close(fd);
-    filename = tmpfilename;
-  }
-  else
-    filename = argv[6];
-
  /*
   * Extract the hostname and printer name from the URI...
   */
 
-  httpSeparateURI(HTTP_URI_CODING_ALL, cupsBackendDeviceURI(argv),
-                  method, sizeof(method), username, sizeof(username),
-		  hostname, sizeof(hostname), &port,
+  if ((device_uri = cupsBackendDeviceURI(argv)) == NULL)
+    return (CUPS_BACKEND_FAILED);
+
+  httpSeparateURI(HTTP_URI_CODING_ALL, device_uri, scheme, sizeof(scheme),
+                  username, sizeof(username), hostname, sizeof(hostname), &port,
 		  resource, sizeof(resource));
 
+  if (!port)
+    port = 515;				/* Default to port 515 */
+
   if (!username[0])
   {
    /*
@@ -249,6 +213,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   * See if there are any options...
   */
 
+  mode          = MODE_STANDARD;
   banner        = 0;
   format        = 'l';
   order         = ORDER_CONTROL_DATA;
@@ -258,10 +223,16 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   contimeout    = 7 * 24 * 60 * 60;
 
 #ifdef __APPLE__
-  /* We want to pass utf-8 characters, not re-map them (3071945) */
+ /*
+  * We want to pass utf-8 characters, not re-map them (3071945)
+  */
+
   sanitize_title = 0;
 
-  /* Get the default timeout from a system preference... */
+ /*
+  * Get the default timeout from a system preference...
+  */
+
   {
     CFPropertyListRef	pvalue;		/* Preference value */
     SInt32		toval;		/* Timeout value */
@@ -305,29 +276,30 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       * Get the name...
       */
 
-      for (ptr = name; *options && *options != '=';)
-        if (ptr < (name + sizeof(name) - 1))
-          *ptr++ = *options++;
-      *ptr = '\0';
+      name = options;
 
-      if (*options == '=')
+      while (*options && *options != '=' && *options != '+' && *options != '&')
+        options ++;
+
+      if ((sep = *options) != '\0')
+        *options++ = '\0';
+
+      if (sep == '=')
       {
        /*
         * Get the value...
 	*/
 
-        options ++;
+        value = options;
 
-	for (ptr = value; *options && *options != '+' && *options != '&';)
-          if (ptr < (value + sizeof(value) - 1))
-            *ptr++ = *options++;
-	*ptr = '\0';
-
-	if (*options == '+' || *options == '&')
+	while (*options && *options != '+' && *options != '&')
 	  options ++;
+
+        if (*options)
+	  *options++ = '\0';
       }
       else
-        value[0] = '\0';
+        value = (char *)"";
 
      /*
       * Process the option...
@@ -351,7 +323,22 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
         if (strchr("cdfglnoprtv", value[0]))
 	  format = value[0];
 	else
-	  fprintf(stderr, "ERROR: Unknown format character \"%c\"\n", value[0]);
+	  _cupsLangPrintf(stderr, _("ERROR: Unknown format character \"%c\"\n"),
+		          value[0]);
+      }
+      else if (!strcasecmp(name, "mode") && value[0])
+      {
+       /*
+        * Set control/data order...
+	*/
+
+        if (!strcasecmp(value, "standard"))
+	  mode = MODE_STANDARD;
+	else if (!strcasecmp(value, "stream"))
+	  mode = MODE_STREAM;
+	else
+	  _cupsLangPrintf(stderr, _("ERROR: Unknown print mode \"%s\"\n"),
+	                  value);
       }
       else if (!strcasecmp(name, "order") && value[0])
       {
@@ -364,7 +351,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 	else if (!strcasecmp(value, "data,control"))
 	  order = ORDER_DATA_CONTROL;
 	else
-	  fprintf(stderr, "ERROR: Unknown file order \"%s\"\n", value);
+	  _cupsLangPrintf(stderr, _("ERROR: Unknown file order \"%s\"\n"),
+	                  value);
       }
       else if (!strcasecmp(name, "reserve"))
       {
@@ -397,7 +385,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 	*/
 
         sanitize_title = !value[0] || !strcasecmp(value, "on") ||
-	 		!strcasecmp(value, "yes") || !strcasecmp(value, "true");
+	 		 !strcasecmp(value, "yes") || !strcasecmp(value, "true");
       }
       else if (!strcasecmp(name, "timeout"))
       {
@@ -411,7 +399,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       else if (!strcasecmp(name, "contimeout"))
       {
        /*
-        * Set the timeout...
+        * Set the connection timeout...
 	*/
 
 	if (atoi(value) > 0)
@@ -420,6 +408,74 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     }
   }
 
+  if (mode == MODE_STREAM)
+    order = ORDER_CONTROL_DATA;
+
+ /*
+  * If we have 7 arguments, print the file named on the command-line.
+  * Otherwise, copy stdin to a temporary file and print the temporary
+  * file.
+  */
+
+  if (argc == 6 && mode == MODE_STANDARD)
+  {
+   /*
+    * Copy stdin to a temporary file...
+    */
+
+    http_addrlist_t	*addrlist;	/* Address list */
+    int			snmp_fd;	/* SNMP socket */
+
+
+    fputs("STATE: +connecting-to-device\n", stderr);
+    fprintf(stderr, "DEBUG: Looking up \"%s\"...\n", hostname);
+
+    if ((addrlist = httpAddrGetList(hostname, AF_UNSPEC, "1")) == NULL)
+    {
+      _cupsLangPrintf(stderr, _("ERROR: Unable to locate printer \'%s\'!\n"),
+		      hostname);
+      return (CUPS_BACKEND_STOP);
+    }
+
+    snmp_fd = _cupsSNMPOpen(addrlist->addr.addr.sa_family);
+
+    if ((fd = cupsTempFd(tmpfilename, sizeof(tmpfilename))) < 0)
+    {
+      _cupsLangPrintError(_("ERROR: Unable to create temporary file"));
+      return (CUPS_BACKEND_FAILED);
+    }
+
+    _cupsLangPuts(stderr, _("INFO: Copying print data...\n"));
+
+    backendRunLoop(-1, fd, snmp_fd, &(addrlist->addr), 0, backendNetworkSideCB);
+
+    if (snmp_fd >= 0)
+      _cupsSNMPClose(snmp_fd);
+
+    httpAddrFreeList(addrlist);
+  }
+  else if (argc == 6)
+  {
+   /*
+    * Stream from stdin...
+    */
+
+    filename = NULL;
+    fd       = 0;
+  }
+  else
+  {
+    filename = argv[6];
+    fd       = open(filename, O_RDONLY);
+
+    if (fd == -1)
+    {
+      _cupsLangPrintf(stderr, _("ERROR: Unable to open print file %s: %s\n"),
+                      filename, strerror(errno));
+      return (CUPS_BACKEND_FAILED);
+    }
+  }
+
  /*
   * Sanitize the document title...
   */
@@ -433,6 +489,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     * the remote end...
     */
 
+    char *ptr;
+
     for (ptr = title; *ptr; ptr ++)
       if (!isalnum(*ptr & 255) && !isspace(*ptr & 255))
 	*ptr = '_';
@@ -455,7 +513,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       copies        = atoi(argv[4]);
     }
 
-    status = lpd_queue(hostname, port, resource + 1, filename,
+    status = lpd_queue(hostname, port, resource + 1, fd, mode,
                        username, title, copies,
 		       banner, format, order, reserve, manual_copies,
 		       timeout, contimeout);
@@ -464,7 +522,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       fprintf(stderr, "PAGE: 1 %d\n", atoi(argv[4]));
   }
   else
-    status = lpd_queue(hostname, port, resource + 1, filename,
+    status = lpd_queue(hostname, port, resource + 1, fd, mode,
                        username, title, 1,
 		       banner, format, order, reserve, 1,
 		       timeout, contimeout);
@@ -476,6 +534,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   if (tmpfilename[0])
     unlink(tmpfilename);
 
+  if (fd)
+    close(fd);
+
  /*
   * Return the queue status...
   */
@@ -525,7 +586,7 @@ lpd_command(int  fd,		/* I - Socket connection to LPD host */
 
   if (lpd_write(fd, buf, bytes) < bytes)
   {
-    perror("ERROR: Unable to send LPD command");
+    _cupsLangPrintError(_("ERROR: Unable to send LPD command"));
     return (-1);
   }
 
@@ -533,14 +594,15 @@ lpd_command(int  fd,		/* I - Socket connection to LPD host */
   * Read back the status from the command and return it...
   */
 
-  fprintf(stderr, "DEBUG: Reading command status...\n");
+  fputs("DEBUG: Reading command status...\n", stderr);
 
   alarm(timeout);
 
   if (recv(fd, &status, 1, 0) < 1)
   {
-    fprintf(stderr, "WARNING: Remote host did not respond with command "
-	            "status byte after %d seconds!\n", timeout);
+    _cupsLangPrintf(stderr,
+		    _("WARNING: Remote host did not respond with command "
+		      "status byte after %d seconds!\n"), timeout);
     status = errno;
   }
 
@@ -560,7 +622,8 @@ static int				/* O - Zero on success, non-zero on failure */
 lpd_queue(const char *hostname,		/* I - Host to connect to */
           int        port,		/* I - Port to connect on */
           const char *printer,		/* I - Printer/queue name */
-	  const char *filename,		/* I - File to print */
+	  int        print_fd,		/* I - File to print */
+	  int        mode,		/* I - Print mode */
           const char *user,		/* I - Requesting user */
 	  const char *title,		/* I - Job title */
 	  int        copies,		/* I - Number of copies */
@@ -572,7 +635,6 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	  int        timeout,		/* I - Timeout... */
 	  int        contimeout)	/* I - Connection timeout */
 {
-  FILE			*fp;		/* Job file */
   char			localhost[255];	/* Local host name */
   int			error;		/* Error number */
   struct stat		filestats;	/* File statistics */
@@ -582,14 +644,15 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 			*cptr;		/* Pointer into control file string */
   char			status;		/* Status byte from command */
   char			portname[255];	/* Port name */
+  int			delay;		/* Delay for retries... */
   char			addrname[256];	/* Address name */
   http_addrlist_t	*addrlist,	/* Address list */
 			*addr;		/* Socket address */
+  int			snmp_fd,	/* SNMP socket */
+			have_supplies;	/* Printer supports supply levels? */
   int			copy;		/* Copies written */
   time_t		start_time;	/* Time of first connect */
-#ifdef __APPLE__
   int			recoverable;	/* Recoverable error shown? */
-#endif /* __APPLE__ */
   size_t		nbytes;		/* Number of bytes written */
   off_t			tbytes;		/* Total bytes written */
   char			buffer[32768];	/* Output buffer */
@@ -620,20 +683,21 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
   sprintf(portname, "%d", port);
 
+  fputs("STATE: +connecting-to-device\n", stderr);
+  fprintf(stderr, "DEBUG: Looking up \"%s\"...\n", hostname);
+
   if ((addrlist = httpAddrGetList(hostname, AF_UNSPEC, portname)) == NULL)
   {
-    fprintf(stderr, "ERROR: Unable to locate printer \'%s\'!\n",
-            hostname);
+    _cupsLangPrintf(stderr, _("ERROR: Unable to locate printer \'%s\'!\n"),
+                    hostname);
     return (CUPS_BACKEND_STOP);
   }
 
  /*
-  * Remember when we starting trying to connect to the printer...
+  * Remember when we started trying to connect to the printer...
   */
 
-#ifdef __APPLE__
   recoverable = 0;
-#endif /* __APPLE__ */
   start_time  = time(NULL);
 
  /*
@@ -646,11 +710,12 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
     * First try to reserve a port for this connection...
     */
 
-    fputs("STATE: +connecting-to-device\n", stderr);
-    fprintf(stderr, "INFO: Attempting to connect to host %s for printer %s\n",
-            hostname, printer);
+    fprintf(stderr, "DEBUG: Connecting to %s:%d for printer %s\n", hostname,
+            port, printer);
+    _cupsLangPuts(stderr, _("INFO: Connecting to printer...\n"));
 
-    for (lport = reserve == RESERVE_RFC1179 ? 732 : 1024, addr = addrlist;;
+    for (lport = reserve == RESERVE_RFC1179 ? 732 : 1024, addr = addrlist,
+             delay = 5;;
          addr = addr->next)
     {
      /*
@@ -690,7 +755,7 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
 	if ((fd = socket(addr->addr.addr.sa_family, SOCK_STREAM, 0)) < 0)
 	{
-          perror("ERROR: Unable to create socket");
+          _cupsLangPrintError(_("ERROR: Unable to create socket"));
 	  sleep(1);
 
           continue;
@@ -707,7 +772,7 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
 	if ((fd = rresvport_af(&lport, addr->addr.addr.sa_family)) < 0)
 	{
-	  perror("ERROR: Unable to reserve port");
+	  _cupsLangPrintError(_("ERROR: Unable to reserve port"));
 	  sleep(1);
 
 	  continue;
@@ -746,8 +811,9 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	* available printer in the class.
 	*/
 
-        fprintf(stderr, "INFO: Unable to connect to %s, queuing on next printer in class...\n",
-		hostname);
+        _cupsLangPuts(stderr,
+	              _("INFO: Unable to contact printer, queuing on next "
+			"printer in class...\n"));
 
         httpAddrFreeList(addrlist);
 
@@ -765,23 +831,20 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       {
         if (contimeout && (time(NULL) - start_time) > contimeout)
 	{
-	  fputs("ERROR: Printer not responding!\n", stderr);
+	  _cupsLangPuts(stderr, _("ERROR: Printer not responding!\n"));
 	  return (CUPS_BACKEND_FAILED);
 	}
 
-#ifdef __APPLE__
         recoverable = 1;
-	fprintf(stderr, "WARNING: recoverable: "
-	                "Network host \'%s\' is busy, down, or "
-	                "unreachable; will retry in 30 seconds...\n",
-                hostname);
-#else
-	fprintf(stderr, "WARNING: "
-	                "Network host \'%s\' is busy, down, or "
-	                "unreachable; will retry in 30 seconds...\n",
-                hostname);
-#endif /* __APPLE__ */
-	sleep(30);
+
+	_cupsLangPrintf(stderr,
+			_("WARNING: recoverable: Network host \'%s\' is busy; "
+			  "will retry in %d seconds...\n"), hostname, delay);
+
+	sleep(delay);
+
+	if (delay < 30)
+	  delay += 5;
       }
       else if (error == EADDRINUSE)
       {
@@ -793,19 +856,16 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       }
       else
       {
-#ifdef __APPLE__
         recoverable = 1;
-	perror("ERROR: recoverable: "
-	       "Unable to connect to printer; will retry in 30 seconds...");
-#else
-	perror("ERROR: "
-	       "Unable to connect to printer; will retry in 30 seconds...");
-#endif /* __APPLE__ */
-        sleep(30);
+
+        fprintf(stderr, "DEBUG: Connection error: %s\n", strerror(errno));
+	_cupsLangPuts(stderr,
+	              _("ERROR: recoverable: Unable to connect to printer; "
+		        "will retry in 30 seconds...\n"));
+	sleep(30);
       }
     }
 
-#ifdef __APPLE__
     if (recoverable)
     {
      /*
@@ -817,45 +877,70 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       fputs("INFO: recovered: \n", stderr);
       sleep(5);
     }
-#endif /* __APPLE__ */
 
     fputs("STATE: -connecting-to-device\n", stderr);
-    fprintf(stderr, "INFO: Connected to %s...\n", hostname);
+    _cupsLangPuts(stderr, _("INFO: Connected to printer...\n"));
 
 #ifdef AF_INET6
     if (addr->addr.addr.sa_family == AF_INET6)
       fprintf(stderr, "DEBUG: Connected to [%s]:%d (IPv6) (local port %d)...\n", 
-		      httpAddrString(&addr->addr, addrname, sizeof(addrname)),
-		      ntohs(addr->addr.ipv6.sin6_port), lport);
+	      httpAddrString(&addr->addr, addrname, sizeof(addrname)),
+	      ntohs(addr->addr.ipv6.sin6_port), lport);
     else
 #endif /* AF_INET6 */
       if (addr->addr.addr.sa_family == AF_INET)
 	fprintf(stderr, "DEBUG: Connected to %s:%d (IPv4) (local port %d)...\n",
-			httpAddrString(&addr->addr, addrname, sizeof(addrname)),
-			ntohs(addr->addr.ipv4.sin_port), lport);
+		httpAddrString(&addr->addr, addrname, sizeof(addrname)),
+		ntohs(addr->addr.ipv4.sin_port), lport);
+
+   /*
+    * See if the printer supports SNMP...
+    */
+
+    if ((snmp_fd = _cupsSNMPOpen(addr->addr.addr.sa_family)) >= 0)
+      have_supplies = !backendSNMPSupplies(snmp_fd, &(addr->addr), NULL, NULL);
+    else
+      have_supplies = 0;
+
+   /*
+    * Check for side-channel requests...
+    */
+
+    backendCheckSideChannel(snmp_fd, &(addr->addr));
 
    /*
     * Next, open the print file and figure out its size...
     */
 
-    if (stat(filename, &filestats))
+    if (print_fd)
     {
-      httpAddrFreeList(addrlist);
-      close(fd);
+     /*
+      * Use the size from the print file...
+      */
 
-      perror("ERROR: unable to stat print file");
-      return (CUPS_BACKEND_FAILED);
+      if (fstat(print_fd, &filestats))
+      {
+	httpAddrFreeList(addrlist);
+	close(fd);
+
+	_cupsLangPrintError(_("ERROR: unable to stat print file"));
+	return (CUPS_BACKEND_FAILED);
+      }
+
+      filestats.st_size *= manual_copies;
     }
-
-    filestats.st_size *= manual_copies;
-
-    if ((fp = fopen(filename, "rb")) == NULL)
+    else
     {
-      httpAddrFreeList(addrlist);
-      close(fd);
+     /*
+      * Use a "very large value" for the size so that the printer will
+      * keep printing until we close the connection...
+      */
 
-      perror("ERROR: unable to open print file for reading");
-      return (CUPS_BACKEND_FAILED);
+#ifdef _LARGEFILE_SOURCE
+      filestats.st_size = (size_t)(999999999999.0);
+#else
+      filestats.st_size = 2147483647;
+#endif /* _LARGEFILE_SOURCE */
     }
 
    /*
@@ -906,6 +991,16 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
     if (order == ORDER_CONTROL_DATA)
     {
+     /*
+      * Check for side-channel requests...
+      */
+
+      backendCheckSideChannel(snmp_fd, &(addr->addr));
+
+     /*
+      * Send the control file...
+      */
+
       if (lpd_command(fd, timeout, "\002%d cfA%03.3d%.15s\n", strlen(control),
                       (int)getpid() % 1000, localhost))
       {
@@ -915,13 +1010,13 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
         return (CUPS_BACKEND_FAILED);
       }
 
-      fprintf(stderr, "INFO: Sending control file (%u bytes)\n",
-              (unsigned)strlen(control));
+      _cupsLangPrintf(stderr, _("INFO: Sending control file (%u bytes)\n"),
+		      (unsigned)strlen(control));
 
       if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
       {
 	status = errno;
-	perror("ERROR: Unable to write control file");
+	_cupsLangPrintError(_("ERROR: Unable to write control file"));
       }
       else
       {
@@ -929,8 +1024,9 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
         if (read(fd, &status, 1) < 1)
 	{
-	  fprintf(stderr, "WARNING: Remote host did not respond with control "
-	                  "status byte after %d seconds!\n", timeout);
+	  _cupsLangPrintf(stderr,
+			  _("WARNING: Remote host did not respond with control "
+			    "status byte after %d seconds!\n"), timeout);
 	  status = errno;
 	}
 
@@ -938,16 +1034,23 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       }
 
       if (status != 0)
-	fprintf(stderr, "ERROR: Remote host did not accept control file (%d)\n",
-        	status);
+	_cupsLangPrintf(stderr,
+			_("ERROR: Remote host did not accept control file "
+			  "(%d)\n"), status);
       else
-	fputs("INFO: Control file sent successfully\n", stderr);
+	_cupsLangPuts(stderr, _("INFO: Control file sent successfully\n"));
     }
     else
       status = 0;
 
     if (status == 0)
     {
+     /*
+      * Check for side-channel requests...
+      */
+
+      backendCheckSideChannel(snmp_fd, &(addr->addr));
+
      /*
       * Send the print file...
       */
@@ -962,22 +1065,28 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
         return (CUPS_BACKEND_FAILED);
       }
 
-      fprintf(stderr, "INFO: Sending data file (" CUPS_LLFMT " bytes)\n",
-              CUPS_LLCAST filestats.st_size);
+      _cupsLangPrintf(stderr,
+#ifdef HAVE_LONG_LONG
+		      _("INFO: Sending data file (%lld bytes)\n"),
+#else
+		      _("INFO: Sending data file (%ld bytes)\n"),
+#endif /* HAVE_LONG_LONG */
+		      CUPS_LLCAST filestats.st_size);
 
       tbytes = 0;
       for (copy = 0; copy < manual_copies; copy ++)
       {
-	rewind(fp);
+	lseek(print_fd, 0, SEEK_SET);
 
-	while ((nbytes = fread(buffer, 1, sizeof(buffer), fp)) > 0)
+	while ((nbytes = read(print_fd, buffer, sizeof(buffer))) > 0)
 	{
-	  fprintf(stderr, "INFO: Spooling LPR job, %.0f%% complete...\n",
-        	  100.0 * tbytes / filestats.st_size);
+	  _cupsLangPrintf(stderr,
+	                  _("INFO: Spooling LPR job, %.0f%% complete...\n"),
+			  100.0 * tbytes / filestats.st_size);
 
 	  if (lpd_write(fd, buffer, nbytes) < nbytes)
 	  {
-            perror("ERROR: Unable to send print file to printer");
+            _cupsLangPrintError(_("ERROR: Unable to send print file to printer"));
             break;
 	  }
 	  else
@@ -985,43 +1094,60 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	}
       }
 
-      if (tbytes < filestats.st_size)
-	status = errno;
-      else if (lpd_write(fd, "", 1) < 1)
+      if (mode == MODE_STANDARD)
       {
-        perror("ERROR: Unable to send trailing nul to printer");
-	status = errno;
+	if (tbytes < filestats.st_size)
+	  status = errno;
+	else if (lpd_write(fd, "", 1) < 1)
+	{
+          _cupsLangPrintError(_("ERROR: Unable to send trailing nul to printer"));
+	  status = errno;
+	}
+	else
+	{
+	 /*
+          * Read the status byte from the printer; if we can't read the byte
+	  * back now, we should set status to "errno", however at this point
+	  * we know the printer got the whole file and we don't necessarily
+	  * want to requeue it over and over...
+	  */
+
+	  alarm(timeout);
+
+          if (recv(fd, &status, 1, 0) < 1)
+	  {
+	    _cupsLangPrintf(stderr,
+			    _("WARNING: Remote host did not respond with data "
+			      "status byte after %d seconds!\n"), timeout);
+	    status = 0;
+          }
+
+	  alarm(0);
+	}
       }
       else
-      {
-       /*
-        * Read the status byte from the printer; if we can't read the byte
-	* back now, we should set status to "errno", however at this point
-	* we know the printer got the whole file and we don't necessarily
-	* want to requeue it over and over...
-	*/
-
-	alarm(timeout);
-
-        if (recv(fd, &status, 1, 0) < 1)
-	{
-	  fprintf(stderr, "WARNING: Remote host did not respond with data "
-	                  "status byte after %d seconds!\n", timeout);
-	  status = 0;
-        }
-
-	alarm(0);
-      }
+        status = 0;
 
       if (status != 0)
-	fprintf(stderr, "ERROR: Remote host did not accept data file (%d)\n",
-        	status);
+	_cupsLangPrintf(stderr,
+	                _("ERROR: Remote host did not accept data file (%d)\n"),
+			status);
       else
-	fputs("INFO: Data file sent successfully\n", stderr);
+	_cupsLangPuts(stderr, _("INFO: Data file sent successfully\n"));
     }
 
     if (status == 0 && order == ORDER_DATA_CONTROL)
     {
+     /*
+      * Check for side-channel requests...
+      */
+
+      backendCheckSideChannel(snmp_fd, &(addr->addr));
+
+     /*
+      * Send control file...
+      */
+
       if (lpd_command(fd, timeout, "\002%d cfA%03.3d%.15s\n", strlen(control),
                       (int)getpid() % 1000, localhost))
       {
@@ -1031,13 +1157,13 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
         return (CUPS_BACKEND_FAILED);
       }
 
-      fprintf(stderr, "INFO: Sending control file (%lu bytes)\n",
-              (unsigned long)strlen(control));
+      _cupsLangPrintf(stderr, _("INFO: Sending control file (%lu bytes)\n"),
+		      (unsigned long)strlen(control));
 
       if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
       {
 	status = errno;
-	perror("ERROR: Unable to write control file");
+	_cupsLangPrintError(_("ERROR: Unable to write control file"));
       }
       else
       {
@@ -1045,8 +1171,9 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
         if (read(fd, &status, 1) < 1)
 	{
-	  fprintf(stderr, "WARNING: Remote host did not respond with control "
-	                  "status byte after %d seconds!\n", timeout);
+	  _cupsLangPrintf(stderr,
+			  _("WARNING: Remote host did not respond with control "
+			    "status byte after %d seconds!\n"), timeout);
 	  status = errno;
 	}
 
@@ -1054,18 +1181,25 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       }
 
       if (status != 0)
-	fprintf(stderr, "ERROR: Remote host did not accept control file (%d)\n",
-        	status);
+	_cupsLangPrintf(stderr,
+			_("ERROR: Remote host did not accept control file "
+			  "(%d)\n"), status);
       else
-	fputs("INFO: Control file sent successfully\n", stderr);
+	_cupsLangPuts(stderr, _("INFO: Control file sent successfully\n"));
     }
 
+   /*
+    * Collect the final supply levels as needed...
+    */
+
+    if (have_supplies)
+      backendSNMPSupplies(snmp_fd, &(addr->addr), NULL, NULL);
+
    /*
     * Close the socket connection and input file...
     */
 
     close(fd);
-    fclose(fp);
 
     if (status == 0)
     {
@@ -1242,5 +1376,5 @@ sigterm_handler(int sig)		/* I - Signal */
 
 
 /*
- * End of "$Id: lpd.c 6061 2006-10-23 00:26:52Z mike $".
+ * End of "$Id: lpd.c 8731 2009-06-26 18:26:36Z mike $".
  */
