@@ -1,9 +1,9 @@
 /*
- * "$Id: parallel.c 7021 2007-10-10 22:50:07Z mike $"
+ * "$Id: parallel.c 8807 2009-08-31 18:45:43Z mike $"
  *
  *   Parallel port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -59,7 +59,8 @@
  */
 
 static void	list_devices(void);
-static void	side_cb(int print_fd, int device_fd, int use_bc);
+static int	side_cb(int print_fd, int device_fd, int snmp_fd,
+		        http_addr_t *addr, int use_bc);
 
 
 /*
@@ -146,7 +147,9 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
     if ((print_fd = open(argv[6], O_RDONLY)) < 0)
     {
-      perror("ERROR: unable to open print file");
+      _cupsLangPrintf(stderr,
+                      _("ERROR: Unable to open print file \"%s\": %s\n"),
+		      argv[6], strerror(errno));
       return (CUPS_BACKEND_FAILED);
     }
 
@@ -281,7 +284,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       lseek(print_fd, 0, SEEK_SET);
     }
 
-    tbytes = backendRunLoop(print_fd, device_fd, use_bc, side_cb);
+    tbytes = backendRunLoop(print_fd, device_fd, -1, NULL, use_bc, side_cb);
 
     if (print_fd != 0 && tbytes >= 0)
       _cupsLangPrintf(stderr,
@@ -325,6 +328,7 @@ list_devices(void)
 	basedevice[255],	/* Base device filename for ports */
 	device_id[1024],	/* Device ID string */
 	make_model[1024],	/* Make and model */
+	info[1024],		/* Info string */
 	uri[1024];		/* Device URI */
 
 
@@ -356,10 +360,15 @@ list_devices(void)
       if (!backendGetDeviceID(fd, device_id, sizeof(device_id),
                               make_model, sizeof(make_model),
 			      NULL, uri, sizeof(uri)))
-	printf("direct %s \"%s\" \"%s LPT #%d\" \"%s\"\n", uri,
-	       make_model, make_model, i + 1, device_id);
+      {
+        snprintf(info, sizeof(info), "%s LPT #%d", make_model, i + 1);
+	cupsBackendReport("direct", uri, make_model, info, device_id, NULL);
+      }
       else
-	printf("direct %s \"Unknown\" \"LPT #%d\"\n", uri, i + 1);
+      {
+        snprintf(info, sizeof(info), "LPT #%d", i + 1);
+	cupsBackendReport("direct", uri, NULL, info, NULL, NULL);
+      }
 
       close(fd);
     }
@@ -561,7 +570,7 @@ list_devices(void)
       printf("direct parallel:%s \"Unknown\" \"Parallel Port #%d\"\n", device, i + 1);
     }
   }
-#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__) || defined(__FreeBSD_kernel__)
   int	i;			/* Looping var */
   int	fd;			/* File descriptor */
   char	device[255];		/* Device filename */
@@ -606,10 +615,12 @@ list_devices(void)
  * 'side_cb()' - Handle side-channel requests...
  */
 
-static void
-side_cb(int print_fd,			/* I - Print file */
-        int device_fd,			/* I - Device file */
-	int use_bc)			/* I - Using back-channel? */
+static int				/* O - 0 on success, -1 on error */
+side_cb(int         print_fd,		/* I - Print file */
+        int         device_fd,		/* I - Device file */
+        int         snmp_fd,		/* I - SNMP socket (unused) */
+	http_addr_t *addr,		/* I - Device address (unused) */
+	int         use_bc)		/* I - Using back-channel? */
 {
   cups_sc_command_t	command;	/* Request command */
   cups_sc_status_t	status;		/* Request/response status */
@@ -617,13 +628,13 @@ side_cb(int print_fd,			/* I - Print file */
   int			datalen;	/* Request/response data size */
 
 
+  (void)snmp_fd;
+  (void)addr;
+
   datalen = sizeof(data);
 
   if (cupsSideChannelRead(&command, &status, data, &datalen, 1.0))
-  {
-    _cupsLangPuts(stderr, _("WARNING: Failed to read side-channel request!\n"));
-    return;
-  }
+    return (-1);
 
   switch (command)
   {
@@ -639,6 +650,7 @@ side_cb(int print_fd,			/* I - Print file */
         break;
 
     case CUPS_SC_CMD_GET_BIDI :
+	status  = CUPS_SC_STATUS_OK;
         data[0] = use_bc;
         datalen = 1;
         break;
@@ -665,10 +677,10 @@ side_cb(int print_fd,			/* I - Print file */
 	break;
   }
 
-  cupsSideChannelWrite(command, status, data, datalen, 1.0);
+  return (cupsSideChannelWrite(command, status, data, datalen, 1.0));
 }
 
 
 /*
- * End of "$Id: parallel.c 7021 2007-10-10 22:50:07Z mike $".
+ * End of "$Id: parallel.c 8807 2009-08-31 18:45:43Z mike $".
  */
