@@ -1,7 +1,7 @@
 /*
-* "$Id: usb-darwin.c 8207 2009-01-07 17:51:00Z mike $"
+* "$Id: usb-darwin.c 8855 2009-10-26 17:20:01Z mike $"
 *
-* Copyright 2005-2008 Apple Inc. All rights reserved.
+* Copyright 2005-2009 Apple Inc. All rights reserved.
 *
 * IMPORTANT:  This Apple software is supplied to you by Apple Computer,
 * Inc. ("Apple") in consideration of your agreement to the following
@@ -65,7 +65,7 @@
 *  cfstr_create_trim()	- Create CFString and trim whitespace characters.
 *  parse_options()	- Parse uri options.
 *  setup_cfLanguage()	- Create AppleLanguages array from LANG environment var.
-*  run_ppc_backend()	- Re-exec i386 backend as ppc.
+*  run_legacy_backend()	- Re-exec backend as ppc or i386.
 *  sigterm_handler()	- SIGTERM handler.
 *  next_line()		- Find the next line in a buffer.
 *  parse_pserror()	- Scan the backchannel data for postscript errors.
@@ -98,8 +98,19 @@
 #include <IOKit/usb/IOUSBLib.h>
 #include <IOKit/IOCFPlugIn.h>
 
+#include <spawn.h>
 #include <pthread.h>
 
+extern char **environ;
+
+
+/*
+ * DEBUG_WRITES, if defined, causes the backend to write data to the printer in
+ * 512 byte increments, up to 8192 bytes, to make debugging with a USB bus
+ * analyzer easier.
+ */
+
+#define DEBUG_WRITES 0
 
 /*
  * WAIT_EOF_DELAY is number of seconds we'll wait for responses from
@@ -229,6 +240,9 @@ typedef struct globals_s
 
   int			print_fd;	/* File descriptor to print */
   ssize_t		print_bytes;	/* Print bytes read */
+#if DEBUG_WRITES
+  ssize_t		debug_bytes;	/* Current bytes to read */
+#endif /* DEBUG_WRITES */
 
   Boolean		wait_eof;
   int			drain_output;	/* Drain all pending output */
@@ -275,11 +289,11 @@ static void setup_cfLanguage(void);
 static void soft_reset();
 static void status_timer_cb(CFRunLoopTimerRef timer, void *info);
 
-#if defined(__i386__)
-static pid_t	child_pid;					/* Child PID */
-static void run_ppc_backend(int argc, char *argv[], int fd);	/* Starts child backend process running as a ppc executable */
-static void sigterm_handler(int sig);				/* SIGTERM handler */
-#endif /* __i386__ */
+#if defined(__i386__) || defined(__x86_64__)
+static pid_t	child_pid;		/* Child PID */
+static void run_legacy_backend(int argc, char *argv[], int fd);	/* Starts child backend process running as a ppc executable */
+static void sigterm_handler(int sig);	/* SIGTERM handler */
+#endif /* __i386__ || __x86_64__ */
 
 #ifdef PARSE_PS_ERRORS
 static const char *next_line (const char *buffer);
@@ -314,6 +328,7 @@ print_device(const char *uri,		/* I - Device URI */
 {
   char		  serial[1024];		/* Serial number buffer */
   OSStatus	  status;		/* Function results */
+  IOReturn	  iostatus;		/* Current IO status */
   pthread_t	  read_thread_id,	/* Read thread */
 		  sidechannel_thread_id;/* Side-channel thread */
   int		  have_sidechannel = 0;	/* Was the side-channel thread started? */
@@ -358,9 +373,14 @@ print_device(const char *uri,		/* I - Device URI */
 
   if (!g.make || !g.model)
   {
-    _cupsLangPrintf(stderr, 
-		    _("ERROR: Unable to create make and model strings\n"));
-    return CUPS_BACKEND_STOP;
+    _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
+
+    if (!g.make)
+      fputs("DEBUG: USB make string is NULL!\n", stderr);
+    if (!g.model)
+      fputs("DEBUG: USB model string is NULL!\n", stderr);
+
+    return (CUPS_BACKEND_STOP);
   }
 
   fputs("STATE: +connecting-to-device\n", stderr);
@@ -387,17 +407,18 @@ print_device(const char *uri,		/* I - Device URI */
 
     status = registry_open(&driverBundlePath);
 
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
     /*
-     * If we were unable to load the class drivers for this printer it's probably because they're ppc-only.
-     * In this case try to fork & exec this backend as a ppc executable so we can use them...
+     * If we were unable to load the class drivers for this printer it's
+     * probably because they're ppc or i386. In this case try to run this
+     * backend as i386 or ppc executables so we can use them...
      */
     if (status == -2)
     {
-      run_ppc_backend(argc, argv, print_fd);
+      run_legacy_backend(argc, argv, print_fd);
       /* Never returns here */
     }
-#endif /* __i386__ */
+#endif /* __i386__ || __x86_64__ */
 
     if (status ==  -2)
     {
@@ -410,12 +431,13 @@ print_device(const char *uri,		/* I - Device URI */
         strlcpy(print_buffer, "USB class driver", sizeof(print_buffer));
 
       fputs("STATE: +apple-missing-usbclassdriver-error\n", stderr);
-      _cupsLangPrintf(stderr, _("FATAL: Could not load %s\n"), print_buffer);
+      _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
+      fprintf(stderr, "DEBUG: Could not load %s\n", print_buffer);
 
       if (driverBundlePath)
 	CFRelease(driverBundlePath);
 
-      return CUPS_BACKEND_STOP;
+      return (CUPS_BACKEND_STOP);
     }
 
     if (driverBundlePath)
@@ -427,8 +449,9 @@ print_device(const char *uri,		/* I - Device URI */
       countdown -= PRINTER_POLLING_INTERVAL;
       if (countdown <= 0)
       {
-	_cupsLangPrintf(stderr, _("INFO: Printer busy (status:0x%08x)\n"),
-	                (int)status);
+	_cupsLangPuts(stderr,
+		      _("INFO: Waiting for printer to become available...\n"));
+	fprintf(stderr, "DEBUG: USB printer status: 0x%08x\n", (int)status);
 	countdown = SUBSEQUENT_LOG_INTERVAL;	/* subsequent log entries, every 15 seconds */
       }
     }
@@ -473,8 +496,9 @@ print_device(const char *uri,		/* I - Device URI */
 
     if (pthread_create(&sidechannel_thread_id, NULL, sidechannel_thread, NULL))
     {
-      _cupsLangPuts(stderr, _("WARNING: Couldn't create side channel\n"));
-      return CUPS_BACKEND_STOP;
+      _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
+      fputs("DEBUG: Couldn't create side-channel thread!\n", stderr);
+      return (CUPS_BACKEND_STOP);
     }
   }
 
@@ -490,8 +514,9 @@ print_device(const char *uri,		/* I - Device URI */
 
   if (pthread_create(&read_thread_id, NULL, read_thread, NULL))
   {
-    _cupsLangPuts(stderr, _("WARNING: Couldn't create read channel\n"));
-    return CUPS_BACKEND_STOP;
+    _cupsLangPuts(stderr, _("ERROR: Fatal USB error!\n"));
+    fputs("DEBUG: Couldn't create read thread!\n", stderr);
+    return (CUPS_BACKEND_STOP);
   }
 
  /*
@@ -505,11 +530,11 @@ print_device(const char *uri,		/* I - Device URI */
 
   while (status == noErr && copies-- > 0)
   {
-    _cupsLangPuts(stderr, _("INFO: Sending data\n"));
+    _cupsLangPuts(stderr, _("INFO: Sending print data...\n"));
 
     if (print_fd != STDIN_FILENO)
     {
-      fputs("PAGE: 1 1", stderr);
+      fputs("PAGE: 1 1\n", stderr);
       lseek(print_fd, 0, SEEK_SET);
     }
 
@@ -569,12 +594,13 @@ print_device(const char *uri,		/* I - Device URI */
 	{
 	  fputs("DEBUG: Received an interrupt before any bytes were "
 	        "written, aborting!\n", stderr);
-          return (0);
+          return (CUPS_BACKEND_OK);
 	}
-	else if (errno != EAGAIN)
+	else if (errno != EAGAIN && errno != EINTR)
 	{
-	 _cupsLangPrintf(stderr, _("ERROR: select() returned %d\n"), (int)errno);
-	 return CUPS_BACKEND_STOP;
+	  _cupsLangPuts(stderr, _("ERROR: Unable to read print data!\n"));
+	  perror("DEBUG: select");
+	  return (CUPS_BACKEND_FAILED);
 	}
       }
 
@@ -595,7 +621,16 @@ print_device(const char *uri,		/* I - Device URI */
 
       if (FD_ISSET(print_fd, &input_set))
       {
+#if DEBUG_WRITES
+	g.debug_bytes += 512;
+        if (g.debug_bytes > sizeof(print_buffer))
+	  g.debug_bytes = 512;
+
+	g.print_bytes = read(print_fd, print_buffer, g.debug_bytes);
+
+#else
 	g.print_bytes = read(print_fd, print_buffer, sizeof(print_buffer));
+#endif /* DEBUG_WRITES */
 
 	if (g.print_bytes < 0)
 	{
@@ -603,10 +638,11 @@ print_device(const char *uri,		/* I - Device URI */
 	  * Read error - bail if we don't see EAGAIN or EINTR...
 	  */
 
-	  if (errno != EAGAIN || errno != EINTR)
+	  if (errno != EAGAIN && errno != EINTR)
 	  {
-	    perror("ERROR: Unable to read print data");
-	    return CUPS_BACKEND_STOP;
+	    _cupsLangPuts(stderr, _("ERROR: Unable to read print data!\n"));
+	    perror("DEBUG: read");
+	    return (CUPS_BACKEND_FAILED);
 	  }
 
 	  g.print_bytes = 0;
@@ -628,36 +664,77 @@ print_device(const char *uri,		/* I - Device URI */
 
       if (g.print_bytes)
       {
-	bytes = g.print_bytes;
-
-	status = (*g.classdriver)->WritePipe(g.classdriver, (UInt8*)print_ptr, &bytes, 0);
+	bytes    = g.print_bytes;
+	iostatus = (*g.classdriver)->WritePipe(g.classdriver, (UInt8*)print_ptr, &bytes, 0);
 
        /*
 	* Ignore timeout errors, but retain the number of bytes written to
 	* avoid sending duplicate data (<rdar://problem/6254911>)...
 	*/
 
-	if (status == kIOUSBTransactionTimeout)
-	  status = 0;
+	if (iostatus == kIOUSBTransactionTimeout)
+	{
+	  fputs("DEBUG: Got USB transaction timeout during write!\n", stderr);
+	  iostatus = 0;
+	}
 
-	if (status || bytes < 0)
+       /*
+        * If we've stalled, retry the write...
+	*/
+
+	else if (iostatus == kIOUSBPipeStalled)
+	{
+	  fputs("DEBUG: Got USB pipe stalled during write!\n", stderr);
+
+	  bytes    = g.print_bytes;
+	  iostatus = (*g.classdriver)->WritePipe(g.classdriver, (UInt8*)print_ptr, &bytes, 0);
+	}
+
+       /*
+	* Retry a write after an aborted write since we probably just got
+	* SIGTERM (<rdar://problem/6860126>)...
+	*/
+
+	else if (iostatus == kIOReturnAborted)
+	{
+	  fputs("DEBUG: Got USB return aborted during write!\n", stderr);
+
+	  IOReturn err = (*g.classdriver)->Abort(g.classdriver);
+	  fprintf(stderr, "DEBUG: USB class driver Abort returned %x\n", err);
+
+#if DEBUG_WRITES
+          sleep(5);
+#endif /* DEBUG_WRITES */
+
+	  bytes    = g.print_bytes;
+	  iostatus = (*g.classdriver)->WritePipe(g.classdriver, (UInt8*)print_ptr, &bytes, 0);
+        }
+
+	if (iostatus || bytes < 0)
 	{
 	 /*
 	  * Write error - bail if we don't see an error we can retry...
 	  */
 
-	  OSStatus err = (*g.classdriver)->Abort(g.classdriver);
-	  _cupsLangPrintf(stderr, _("ERROR: %ld: (canceled:%ld)\n"),
-	                  (long)status, (long)err);
-	  status = CUPS_BACKEND_STOP;
+	  _cupsLangPuts(stderr, _("ERROR: Unable to send print data!\n"));
+	  fprintf(stderr, "DEBUG: USB class driver WritePipe returned %x\n",
+	          iostatus);
+
+	  IOReturn err = (*g.classdriver)->Abort(g.classdriver);
+	  fprintf(stderr, "DEBUG: USB class driver Abort returned %x\n",
+	          err);
+
+	  status = CUPS_BACKEND_FAILED;
 	  break;
 	}
+	else if (bytes > 0)
+	{
+	  fprintf(stderr, "DEBUG: Wrote %d bytes of print data...\n", (int)bytes);
 
-        fprintf(stderr, "DEBUG: Wrote %d bytes of print data...\n", (int)bytes);
-
-        g.print_bytes -= bytes;
-	print_ptr   += bytes;
-	total_bytes += bytes;
+	  g.print_bytes -= bytes;
+	  print_ptr   += bytes;
+	  total_bytes += bytes;
+	}
       }
 
       if (print_fd != 0 && status == noErr)
@@ -737,6 +814,7 @@ print_device(const char *uri,		/* I - Device URI */
       * Force the read thread to exit...
       */
 
+      g.wait_eof = 0;
       pthread_kill(read_thread_id, SIGTERM);
     }
   }
@@ -802,6 +880,8 @@ static void *read_thread(void *reference)
     readstatus = (*g.classdriver)->ReadPipe(g.classdriver, readbuffer, &rbytes);
     if (readstatus == kIOReturnSuccess && rbytes > 0)
     {
+      fprintf(stderr, "DEBUG: Read %d bytes of back-channel data...\n",
+              (int)rbytes);
       cupsBackChannelWrite((char*)readbuffer, rbytes, 1.0);
 
       /* cntrl-d is echoed by the printer.
@@ -816,6 +896,12 @@ static void *read_thread(void *reference)
       parse_pserror(readbuffer, rbytes);
 #endif
     }
+    else if (readstatus == kIOUSBTransactionTimeout)
+      fputs("DEBUG: Got USB transaction timeout during read!\n", stderr);
+    else if (readstatus == kIOUSBPipeStalled)
+      fputs("DEBUG: Got USB pipe stalled during read!\n", stderr);
+    else if (readstatus == kIOReturnAborted)
+      fputs("DEBUG: Got USB return aborted during read!\n", stderr);
 
    /*
     * Make sure this loop executes no more than once every 250 miliseconds...
@@ -857,46 +943,88 @@ sidechannel_thread(void *reference)
     datalen = sizeof(data);
 
     if (cupsSideChannelRead(&command, &status, data, &datalen, 1.0))
-      continue;
+      break;
 
     switch (command)
     {
       case CUPS_SC_CMD_SOFT_RESET:	/* Do a soft reset */
+	  fputs("DEBUG: CUPS_SC_CMD_SOFT_RESET received from driver...\n",
+		stderr);
+
           if ((*g.classdriver)->SoftReset != NULL)
 	  {
 	    soft_reset();
 	    cupsSideChannelWrite(command, CUPS_SC_STATUS_OK, NULL, 0, 1.0);
+	    fputs("DEBUG: Returning status CUPS_STATUS_OK with no bytes...\n",
+	          stderr);
 	  }
 	  else
 	  {
 	    cupsSideChannelWrite(command, CUPS_SC_STATUS_NOT_IMPLEMENTED,
 	                         NULL, 0, 1.0);
+	    fputs("DEBUG: Returning status CUPS_STATUS_NOT_IMPLEMENTED with "
+	          "no bytes...\n", stderr);
 	  }
 	  break;
 
       case CUPS_SC_CMD_DRAIN_OUTPUT:	/* Drain all pending output */
+	  fputs("DEBUG: CUPS_SC_CMD_DRAIN_OUTPUT received from driver...\n",
+		stderr);
+
 	  g.drain_output = 1;
 	  break;
 
       case CUPS_SC_CMD_GET_BIDI:		/* Is the connection bidirectional? */
+	  fputs("DEBUG: CUPS_SC_CMD_GET_BIDI received from driver...\n",
+		stderr);
+
 	  data[0] = g.bidi_flag;
 	  cupsSideChannelWrite(command, CUPS_SC_STATUS_OK, data, 1, 1.0);
+
+	  fprintf(stderr,
+	          "DEBUG: Returned CUPS_SC_STATUS_OK with 1 byte (%02X)...\n",
+		  data[0]);
 	  break;
 
       case CUPS_SC_CMD_GET_DEVICE_ID:	/* Return IEEE-1284 device ID */
+	  fputs("DEBUG: CUPS_SC_CMD_GET_DEVICE_ID received from driver...\n",
+		stderr);
+
 	  datalen = sizeof(data);
 	  get_device_id(&status, data, &datalen);
 	  cupsSideChannelWrite(command, CUPS_SC_STATUS_OK, data, datalen, 1.0);
+
+          if (datalen < sizeof(data))
+	    data[datalen] = '\0';
+	  else
+	    data[sizeof(data) - 1] = '\0';
+
+	  fprintf(stderr,
+	          "DEBUG: Returning CUPS_SC_STATUS_OK with %d bytes (%s)...\n",
+		  datalen, data);
 	  break;
 
       case CUPS_SC_CMD_GET_STATE:		/* Return device state */
+	  fputs("DEBUG: CUPS_SC_CMD_GET_STATE received from driver...\n",
+		stderr);
+
 	  data[0] = CUPS_SC_STATE_ONLINE;
 	  cupsSideChannelWrite(command, CUPS_SC_STATUS_OK, data, 1, 1.0);
+
+	  fprintf(stderr,
+	          "DEBUG: Returned CUPS_SC_STATUS_OK with 1 byte (%02X)...\n",
+		  data[0]);
 	  break;
 
       default:
+	  fprintf(stderr, "DEBUG: Unknown side-channel command (%d) received "
+			  "from driver...\n", command);
+
 	  cupsSideChannelWrite(command, CUPS_SC_STATUS_NOT_IMPLEMENTED,
 			       NULL, 0, 1.0);
+
+	  fputs("DEBUG: Returned CUPS_SC_STATUS_NOT_IMPLEMENTED with no bytes...\n",
+		stderr);
 	  break;
     }
   }
@@ -1036,10 +1164,10 @@ static Boolean list_device_cb(void *refcon,
 	snprintf(optionsstr, sizeof(optionsstr), "?location=%x", (unsigned)deviceLocation);
 
       httpAssembleURI(HTTP_URI_CODING_ALL, uristr, sizeof(uristr), "usb", NULL, makestr, 0, modelstr);
-      strncat(uristr, optionsstr, sizeof(uristr));
+      strlcat(uristr, optionsstr, sizeof(uristr));
 
-      printf("direct %s \"%s\" \"%s USB\" \"%s\"\n", uristr, make_modelstr,
-             make_modelstr, idstr);
+      cupsBackendReport("direct", uristr, make_modelstr, make_modelstr, idstr,
+                        NULL);
 
       release_deviceinfo(&make, &model, &serial);
       CFRelease(deviceIDString);
@@ -1121,8 +1249,8 @@ static Boolean find_device_cb(void *refcon,
 
   if (!keepLooking && g.status_timer != NULL)
   {
-    fputs("STATE: -offline-error\n", stderr);
-    _cupsLangPuts(stderr, _("INFO: Printer is now on-line.\n"));
+    fputs("STATE: -offline-report\n", stderr);
+    _cupsLangPuts(stderr, _("INFO: Printer is now online.\n"));
     CFRunLoopRemoveTimer(CFRunLoopGetCurrent(), g.status_timer, kCFRunLoopDefaultMode);
     CFRelease(g.status_timer);
     g.status_timer = NULL;
@@ -1139,8 +1267,8 @@ static Boolean find_device_cb(void *refcon,
 static void status_timer_cb(CFRunLoopTimerRef timer,
 			    void *info)
 {
-  fputs("STATE: +offline-error\n", stderr);
-  _cupsLangPuts(stderr, _("INFO: Printer is currently off-line.\n"));
+  fputs("STATE: +offline-report\n", stderr);
+  _cupsLangPuts(stderr, _("INFO: Printer is offline.\n"));
 
   if (getenv("CLASS") != NULL)
   {
@@ -1222,58 +1350,83 @@ static kern_return_t load_classdriver(CFStringRef	    driverPath,
 				      printer_interface_t   intf,
 				      classdriver_t	    ***printerDriver)
 {
-  kern_return_t kr = kUSBPrinterClassDeviceNotOpen;
-  classdriver_t **driver = NULL;
-  CFStringRef bundle = (driverPath == NULL ? kUSBGenericTOPrinterClassDriver : driverPath);
+  kern_return_t	kr = kUSBPrinterClassDeviceNotOpen;
+  classdriver_t	**driver = NULL;
+  CFStringRef	bundle = driverPath ? driverPath : kUSBGenericTOPrinterClassDriver;
+  char 		bundlestr[1024];	/* Bundle path */
+  struct stat	bundleinfo;		/* File information for bundle */
+  CFURLRef	url;			/* URL for driver */
+  CFPlugInRef	plugin = NULL;		/* Plug-in address */
 
-  if (bundle != NULL)
+
+  CFStringGetCString(bundle, bundlestr, sizeof(bundlestr), kCFStringEncodingUTF8);
+
+ /*
+  * Validate permissions for the class driver...
+  */
+
+  if (stat(bundlestr, &bundleinfo))
   {
-    CFURLRef url = CFURLCreateWithFileSystemPath(NULL, bundle, kCFURLPOSIXPathStyle, true);
-    CFPlugInRef plugin = (url != NULL ? CFPlugInCreate(NULL, url) : NULL);
+    fprintf(stderr, "DEBUG: Unable to load class driver \"%s\": %s\n",
+	    bundlestr, strerror(errno));
+    return (kr);
+  }
+  else if (bundleinfo.st_mode & S_IWOTH)
+  {
+    fprintf(stderr, "DEBUG: Unable to load class driver \"%s\": insecure file "
+		    "permissions (0%o)\n", bundlestr, bundleinfo.st_mode);
+    return (kr);
+  }
 
-    if (url != NULL)
-      CFRelease(url);
+ /*
+  * Try loading the class driver...
+  */
 
-    if (plugin != NULL)
+  url = CFURLCreateWithFileSystemPath(NULL, bundle, kCFURLPOSIXPathStyle, true);
+
+  if (url)
+  {
+    plugin = CFPlugInCreate(NULL, url);
+    CFRelease(url);
+  }
+  else
+    plugin = NULL;
+
+  if (plugin)
+  {
+    CFArrayRef factories = CFPlugInFindFactoriesForPlugInTypeInPlugIn(kUSBPrinterClassTypeID, plugin);
+    if (factories != NULL && CFArrayGetCount(factories) > 0)
     {
-      CFArrayRef factories = CFPlugInFindFactoriesForPlugInTypeInPlugIn(kUSBPrinterClassTypeID, plugin);
-      if (factories != NULL && CFArrayGetCount(factories) > 0)
+      CFUUIDRef factoryID = CFArrayGetValueAtIndex(factories, 0);
+      IUnknownVTbl **iunknown = CFPlugInInstanceCreate(NULL, factoryID, kUSBPrinterClassTypeID);
+      if (iunknown != NULL)
       {
-	CFUUIDRef factoryID = CFArrayGetValueAtIndex(factories, 0);
-	IUnknownVTbl **iunknown = CFPlugInInstanceCreate(NULL, factoryID, kUSBPrinterClassTypeID);
-	if (iunknown != NULL)
+	kr = (*iunknown)->QueryInterface(iunknown, CFUUIDGetUUIDBytes(kUSBPrinterClassInterfaceID), (LPVOID *)&driver);
+	if (kr == kIOReturnSuccess && driver != NULL)
 	{
-	  kr = (*iunknown)->QueryInterface(iunknown, CFUUIDGetUUIDBytes(kUSBPrinterClassInterfaceID), (LPVOID *)&driver);
-	  if (kr == kIOReturnSuccess && driver != NULL)
+	  classdriver_t **genericDriver = NULL;
+	  if (driverPath != NULL && CFStringCompare(driverPath, kUSBGenericTOPrinterClassDriver, 0) != kCFCompareEqualTo)
+	    kr = load_classdriver(NULL, intf, &genericDriver);
+
+	  if (kr == kIOReturnSuccess)
 	  {
-	    classdriver_t **genericDriver = NULL;
-	    if (driverPath != NULL && CFStringCompare(driverPath, kUSBGenericTOPrinterClassDriver, 0) != kCFCompareEqualTo)
-	      kr = load_classdriver(NULL, intf, &genericDriver);
+	    (*driver)->interface = intf;
+	    (*driver)->Initialize(driver, genericDriver);
 
-	    if (kr == kIOReturnSuccess)
-	    {
-	      (*driver)->interface = intf;
-	      (*driver)->Initialize(driver, genericDriver);
-
-	      (*driver)->plugin = plugin;
-	      (*driver)->interface = intf;
-	      *printerDriver = driver;
-	    }
+	    (*driver)->plugin = plugin;
+	    (*driver)->interface = intf;
+	    *printerDriver = driver;
 	  }
-	  (*iunknown)->Release(iunknown);
 	}
-	CFRelease(factories);
+	(*iunknown)->Release(iunknown);
       }
+      CFRelease(factories);
     }
   }
 
-#ifdef DEBUG
-  char bundlestr[1024];
-  CFStringGetCString(bundle, bundlestr, sizeof(bundlestr), kCFStringEncodingUTF8);
   fprintf(stderr, "DEBUG: load_classdriver(%s) (kr:0x%08x)\n", bundlestr, (int)kr);
-#endif /* DEBUG */
 
-  return kr;
+  return (kr);
 }
 
 
@@ -1696,8 +1849,8 @@ static void setup_cfLanguage(void)
     lang[0] = CFStringCreateWithCString(kCFAllocatorDefault, requestedLang, kCFStringEncodingUTF8);
     langArray = CFArrayCreate(kCFAllocatorDefault, (const void **)lang, sizeof(lang) / sizeof(lang[0]), &kCFTypeArrayCallBacks);
 
-    CFPreferencesSetAppValue(CFSTR("AppleLanguages"), langArray, kCFPreferencesCurrentApplication);
-    DEBUG_printf((stderr, "DEBUG: usb: AppleLanguages = \"%s\"\n", requestedLang));
+    CFPreferencesSetValue(CFSTR("AppleLanguages"), langArray, kCFPreferencesCurrentApplication, kCFPreferencesAnyUser, kCFPreferencesAnyHost);
+    fprintf(stderr, "DEBUG: usb: AppleLanguages=\"%s\"\n", requestedLang);
 
     CFRelease(lang[0]);
     CFRelease(langArray);
@@ -1707,36 +1860,42 @@ static void setup_cfLanguage(void)
 }
 
 #pragma mark -
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
 /*!
- * @function	run_ppc_backend
+ * @function	run_legacy_backend
  *
- * @abstract	Starts child backend process running as a ppc executable.
+ * @abstract	Starts child backend process running as a ppc or i386 executable.
  *
  * @result	Never returns; always calls exit().
  *
  * @discussion
  */
-static void run_ppc_backend(int argc,
-			    char *argv[],
-			    int fd)
+static void run_legacy_backend(int argc,
+			       char *argv[],
+			       int fd)
 {
   int	i;
   int	exitstatus = 0;
   int	childstatus;
   pid_t	waitpid_status;
   char	*my_argv[32];
-  char	*usb_ppc_status;
+  char	*usb_legacy_status;
 
-  /*
-   * If we're running as i386 and couldn't load the class driver (because they'it's
-   * ppc-only) then try to re-exec ourselves in ppc mode to try again. If we don't have
-   * a ppc architecture we may be running i386 again so guard against this by setting
-   * and testing an environment variable...
-   */
-  usb_ppc_status = getenv("USB_PPC_STATUS");
+ /*
+  * If we're running as x86_64 or i386 and couldn't load the class driver
+  * (because it's ppc or i386), then try to re-exec ourselves in ppc or i386
+  * mode to try again. If we don't have a ppc or i386 architecture we may be
+  * running with the same architecture again so guard against this by setting
+  * and testing an environment variable...
+  */
 
-  if (usb_ppc_status == NULL)
+#  ifdef __x86_64__
+  usb_legacy_status = getenv("USB_I386_STATUS");
+#  else
+  usb_legacy_status = getenv("USB_PPC_STATUS");
+#  endif /* __x86_64__ */
+
+  if (!usb_legacy_status)
   {
    /*
     * Setup a SIGTERM handler then block it before forking...
@@ -1745,6 +1904,9 @@ static void run_ppc_backend(int argc,
     struct sigaction	action;		/* POSIX signal action */
     sigset_t		newmask,	/* New signal mask */
 			oldmask;	/* Old signal mask */
+    char		usbpath[1024];	/* Path to USB backend */
+    const char		*cups_serverbin;/* Path to CUPS binaries */
+
 
     memset(&action, 0, sizeof(action));
     sigaddset(&action.sa_mask, SIGTERM);
@@ -1755,55 +1917,62 @@ static void run_ppc_backend(int argc,
     sigaddset(&newmask, SIGTERM);
     sigprocmask(SIG_BLOCK, &newmask, &oldmask);
 
-    if ((child_pid = fork()) == 0)
+   /*
+    * Set the environment variable...
+    */
+
+#  ifdef __x86_64__
+    setenv("USB_I386_STATUS", "1", false);
+#  else
+    setenv("USB_PPC_STATUS", "1", false);
+#  endif /* __x86_64__ */
+
+   /*
+    * Tell the kernel to use the specified CPU architecture...
+    */
+
+#  ifdef __x86_64__
+    cpu_type_t cpu = CPU_TYPE_I386;
+#  else
+    cpu_type_t cpu = CPU_TYPE_POWERPC;
+#  endif /* __x86_64__ */
+    size_t ocount = 1;
+    posix_spawnattr_t attrs;
+
+    if (!posix_spawnattr_init(&attrs))
     {
-     /*
-      * Child comes here...
-      */
-
-      setenv("USB_PPC_STATUS", "1", false);
-
-     /*
-      * Unblock signals before doing the exec...
-      */
-
-      memset(&action, 0, sizeof(action));
-      sigemptyset(&action.sa_mask);
-      action.sa_handler = SIG_DFL;
-      sigaction(SIGTERM, &action, NULL);
-
-      sigprocmask(SIG_SETMASK, &oldmask, NULL);
-
-     /*
-      * Tell the kernel the next exec call should favor the ppc architecture...
-      */
-
-      int mib[] = { CTL_KERN, KERN_AFFINITY, 1, 1 };
-      int namelen = 4;
-      sysctl(mib, namelen, NULL, NULL, NULL, 0);
-
-     /*
-      * Set up the arguments and call exec...
-      */
-
-      for (i = 0; i < argc && i < (sizeof(my_argv)/sizeof(my_argv[0])) - 1; i++)
-	my_argv[i] = argv[i];
-
-      my_argv[i] = NULL;
-
-      execv("/usr/libexec/cups/backend/usb", my_argv);
-
-      perror("/usr/libexec/cups/backend/usb");
-      exit(errno);
+      posix_spawnattr_setsigdefault(&attrs, &oldmask);
+      if (posix_spawnattr_setbinpref_np(&attrs, 1, &cpu, &ocount) || ocount != 1)
+      {
+#  ifdef __x86_64__
+	perror("DEBUG: Unable to set binary preference to i386");
+#  else
+	perror("DEBUG: Unable to set binary preference to ppc");
+#  endif /* __x86_64__ */
+	_cupsLangPrintf(stderr, _("Unable to use legacy USB class driver!\n"));
+	exit(CUPS_BACKEND_STOP);
+      }
     }
-    else if (child_pid < 0)
-    {
-     /*
-      * Error - couldn't fork a new process!
-      */
 
-      perror("fork");
-      exit(errno);
+   /*
+    * Set up the arguments and call posix_spawn...
+    */
+
+    if ((cups_serverbin = getenv("CUPS_SERVERBIN")) == NULL)
+      cups_serverbin = CUPS_SERVERBIN;
+    snprintf(usbpath, sizeof(usbpath), "%s/backend/usb", cups_serverbin);
+
+    for (i = 0; i < argc && i < (sizeof(my_argv) / sizeof(my_argv[0])) - 1; i ++)
+      my_argv[i] = argv[i];
+
+    my_argv[i] = NULL;
+
+    if (posix_spawn(&child_pid, usbpath, NULL, &attrs, my_argv, environ))
+    {
+      fprintf(stderr, "DEBUG: Unable to exec %s: %s\n", usbpath,
+              strerror(errno));
+      _cupsLangPrintf(stderr, _("Unable to use legacy USB class driver!\n"));
+      exit(CUPS_BACKEND_STOP);
     }
 
    /*
@@ -1819,28 +1988,33 @@ static void run_ppc_backend(int argc,
     close(fd);
     close(1);
 
-    fprintf(stderr, "DEBUG: Started usb(ppc) backend (PID %d)\n", (int)child_pid);
+    fprintf(stderr, "DEBUG: Started usb(legacy) backend (PID %d)\n",
+            (int)child_pid);
 
     while ((waitpid_status = waitpid(child_pid, &childstatus, 0)) == (pid_t)-1 && errno == EINTR)
       usleep(1000);
 
     if (WIFSIGNALED(childstatus))
     {
-      exitstatus = WTERMSIG(childstatus);
-      fprintf(stderr, "DEBUG: usb(ppc) backend %d crashed on signal %d!\n", child_pid, exitstatus);
+      exitstatus = CUPS_BACKEND_STOP;
+      fprintf(stderr, "DEBUG: usb(legacy) backend %d crashed on signal %d!\n",
+              child_pid, WTERMSIG(childstatus));
     }
     else
     {
       if ((exitstatus = WEXITSTATUS(childstatus)) != 0)
-	fprintf(stderr, "DEBUG: usb(ppc) backend %d stopped with status %d!\n", child_pid, exitstatus);
+	fprintf(stderr,
+	        "DEBUG: usb(legacy) backend %d stopped with status %d!\n",
+		child_pid, exitstatus);
       else
-	fprintf(stderr, "DEBUG: PID %d exited with no errors\n", child_pid);
+	fprintf(stderr, "DEBUG: usb(legacy) backend %d exited with no errors\n",
+	        child_pid);
     }
   }
   else
   {
-    fputs("DEBUG: usb child running i386 again\n", stderr);
-    exitstatus = ENOENT;
+    fputs("DEBUG: usb(legacy) backend running native again\n", stderr);
+    exitstatus = CUPS_BACKEND_STOP;
   }
 
   exit(exitstatus);
@@ -1850,17 +2024,35 @@ static void run_ppc_backend(int argc,
  * 'sigterm_handler()' - SIGTERM handler.
  */
 
-static void sigterm_handler(int sig)
+static void
+sigterm_handler(int sig)		/* I - Signal */
 {
   /* If we started a child process pass the signal on to it...
    */
   if (child_pid)
-    kill(child_pid, sig);
+  {
+   /*
+    * If we started a child process pass the signal on to it...
+    */
 
-  exit(1);
+    int	status;
+
+    kill(child_pid, sig);
+    while (waitpid(child_pid, &status, 0) < 0 && errno == EINTR);
+
+    if (WIFEXITED(status))
+      exit(WEXITSTATUS(status));
+    else if (status == SIGTERM || status == SIGKILL)
+      exit(0);
+    else
+    {
+      fprintf(stderr, "DEBUG: Child crashed on signal %d!\n", status);
+      exit(CUPS_BACKEND_STOP);
+    }
+  }
 }
 
-#endif /* __i386__ */
+#endif /* __i386__ || __x86_64__ */
 
 
 #ifdef PARSE_PS_ERRORS
@@ -2011,12 +2203,11 @@ static void get_device_id(cups_sc_status_t *status,
 			  char *data,
 			  int *datalen)
 {
-  UInt32 deviceLocation = 0;
-  UInt8	interfaceNum = 0;
   CFStringRef deviceIDString = NULL;
 
   /* GetDeviceID */
-  copy_devicestring(g.printer_obj, &deviceIDString, &deviceLocation, &interfaceNum);
+  copy_deviceid(g.classdriver, &deviceIDString);
+
   if (deviceIDString)
   {
     CFStringGetCString(deviceIDString, data, *datalen, kCFStringEncodingUTF8);
@@ -2028,5 +2219,5 @@ static void get_device_id(cups_sc_status_t *status,
 
 
 /*
- * End of "$Id: usb-darwin.c 8207 2009-01-07 17:51:00Z mike $".
+ * End of "$Id: usb-darwin.c 8855 2009-10-26 17:20:01Z mike $".
  */
