@@ -1,25 +1,16 @@
 /*
- * "$Id: cgi.h 4931 2006-01-14 20:37:40Z mike $"
+ * "$Id: cgi.h 8859 2009-11-09 23:01:17Z mike $"
  *
  *   CGI support library definitions.
  *
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 #ifndef _CUPS_CGI_H_
@@ -63,11 +54,13 @@ typedef struct cgi_file_s		/**** Uploaded file data ****/
 extern void		cgiAbort(const char *title, const char *stylesheet,
 			         const char *format, ...);
 extern int		cgiCheckVariables(const char *names);
+extern void		cgiClearVariables(void);
 extern void		*cgiCompileSearch(const char *query);
 extern void		cgiCopyTemplateFile(FILE *out, const char *tmpl);
 extern void		cgiCopyTemplateLang(const char *tmpl);
 extern int		cgiDoSearch(void *search, const char *text);
 extern void		cgiEndHTML(void);
+extern void		cgiEndMultipart(void);
 extern char		*cgiFormEncode(char *dst, const char *src, size_t dstsize);
 extern void		cgiFreeSearch(void *search);
 extern const char	*cgiGetArray(const char *name, int element);
@@ -81,6 +74,8 @@ extern const char	*cgiGetVariable(const char *name);
 extern int		cgiInitialize(void);
 extern int		cgiIsPOST(void);
 extern void		cgiMoveJobs(http_t *http, const char *dest, int job_id);
+extern void		cgiPrintCommand(http_t *http, const char *dest,
+			                const char *command, const char *title);
 extern void		cgiPrintTestPage(http_t *http, const char *dest);
 extern char		*cgiRewriteURL(const char *uri, char *url, int urlsize,
 			               const char *newresource);
@@ -100,10 +95,12 @@ extern void		cgiSetVariable(const char *name, const char *value);
 extern void		cgiShowIPPError(const char *message);
 extern void		cgiShowJobs(http_t *http, const char *dest);
 extern void		cgiStartHTML(const char *title);
+extern void		cgiStartMultipart(void);
+extern int		cgiSupportsMultipart(void);
 extern const char	*cgiText(const char *message);
 
 #endif /* !_CUPS_CGI_H_ */
 
 /*
- * End of "$Id: cgi.h 4931 2006-01-14 20:37:40Z mike $".
+ * End of "$Id: cgi.h 8859 2009-11-09 23:01:17Z mike $".
  */
