@@ -1,5 +1,5 @@
 /*
- * "$Id: testfile.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: testfile.c 8771 2009-08-20 18:14:49Z mike $"
  *
  *   File test program for the Common UNIX Printing System (CUPS).
  *
@@ -17,6 +17,8 @@
  * Contents:
  *
  *   main()             - Main entry.
+ *   count_lines()      - Count the number of lines in a file.
+ *   random_tests()     - Do random access tests.
  *   read_write_tests() - Perform read/write tests.
  */
 
@@ -34,6 +36,11 @@
 #ifdef HAVE_LIBZ
 #  include <zlib.h>
 #endif /* HAVE_LIBZ */
+#ifdef WIN32
+#  include <io.h>
+#else
+#  include <unistd.h>
+#endif /* WIN32 */
 #include <fcntl.h>
 
 
@@ -41,6 +48,7 @@
  * Local functions...
  */
 
+static int	count_lines(cups_file_t *fp);
 static int	random_tests(void);
 static int	read_write_tests(int compression);
 
@@ -55,8 +63,12 @@ main(int  argc,				/* I - Number of command-line arguments */
 {
   int		status;			/* Exit status */
   char		filename[1024];		/* Filename buffer */
+  cups_file_t	*fp;			/* File pointer */
+#ifndef WIN32
   int		fds[2];			/* Open file descriptors */
   cups_file_t	*fdfile;		/* File opened with cupsFileOpenFd() */
+#endif /* !WIN32 */
+  int		count;			/* Number of lines in file */
 
 
   if (argc == 1)
@@ -83,6 +95,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
     status += random_tests();
 
+#ifndef WIN32
    /*
     * Test fdopen and close without reading...
     */
@@ -116,6 +129,56 @@ main(int  argc,				/* I - Number of command-line arguments */
 
       puts("PASS");
     }
+#endif /* !WIN32 */
+
+   /*
+    * Count lines in euc-jp.txt, rewind, then count again.
+    */
+
+    fputs("\ncupsFileOpen(\"../data/euc-jp.txt\", \"r\"): ", stdout);
+
+    if ((fp = cupsFileOpen("../data/euc-jp.txt", "r")) == NULL)
+    {
+      puts("FAIL");
+      status ++;
+    }
+    else
+    {
+      puts("PASS");
+      fputs("cupsFileGets: ", stdout);
+
+      if ((count = count_lines(fp)) != 15184)
+      {
+        printf("FAIL (got %d lines, expected 15184)\n", count);
+	status ++;
+      }
+      else
+      {
+        puts("PASS");
+	fputs("cupsFileRewind: ", stdout);
+
+	if (cupsFileRewind(fp) != 0)
+	{
+	  puts("FAIL");
+	  status ++;
+	}
+	else
+	{
+	  puts("PASS");
+	  fputs("cupsFileGets: ", stdout);
+
+	  if ((count = count_lines(fp)) != 15184)
+	  {
+	    printf("FAIL (got %d lines, expected 15184)\n", count);
+	    status ++;
+	  }
+	  else
+	    puts("PASS");
+        }
+      }
+
+      cupsFileClose(fp);
+    }
 
    /*
     * Test path functions...
@@ -151,10 +214,8 @@ main(int  argc,				/* I - Number of command-line arguments */
     * Cat the filename on the command-line...
     */
 
-    cups_file_t	*fp;			/* File pointer */
     char	line[1024];		/* Line from file */
 
-
     if ((fp = cupsFileOpen(argv[1], "r")) == NULL)
     {
       perror(argv[1]);
@@ -178,6 +239,23 @@ main(int  argc,				/* I - Number of command-line arguments */
 }
 
 
+/*
+ * 'count_lines()' - Count the number of lines in a file.
+ */
+
+static int				/* O - Number of lines */
+count_lines(cups_file_t *fp)		/* I - File to read from */
+{
+  int	count;				/* Number of lines */
+  char	line[1024];			/* Line buffer */
+
+
+  for (count = 0; cupsFileGets(fp, line, sizeof(line)); count ++);
+
+  return (count);
+}
+
+
 /*
  * 'random_tests()' - Do random access tests.
  */
@@ -682,7 +760,7 @@ read_write_tests(int compression)	/* I - Use compression? */
 
     fputs("cupsFileGetChar(partial line): ", stdout);
 
-    for (i = 0; i < strlen(partial_line); i ++)
+    for (i = 0; i < (int)strlen(partial_line); i ++)
       if ((byte = cupsFileGetChar(fp)) < 0)
         break;
       else if (byte != partial_line[i])
@@ -745,5 +823,5 @@ read_write_tests(int compression)	/* I - Use compression? */
 
 
 /*
- * End of "$Id: testfile.c 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: testfile.c 8771 2009-08-20 18:14:49Z mike $".
  */
